library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.numeric_std.all;

Entity enseignement_addition_axi is 
    generic(
        id        : natural := 1;
		-- Parameters of Axi Slave Bus Interface S00_AXI
		C_S00_AXI_DATA_WIDTH	: integer	:= 32;
		C_S00_AXI_ADDR_WIDTH	: integer	:= 5
    );
    port (
		-- axi slave signals
		s00_axi_aclk	: in std_logic;
		s00_axi_reset	: in std_logic;
		s00_axi_awaddr	: in std_logic_vector(C_S00_AXI_ADDR_WIDTH-1 downto 0);
		s00_axi_awprot  : in std_logic_vector(2 downto 0);
		s00_axi_awvalid	: in std_logic;
		s00_axi_awready	: out std_logic;
		s00_axi_wdata	: in std_logic_vector(C_S00_AXI_DATA_WIDTH-1 downto 0);
		s00_axi_wvalid	: in std_logic;
		s00_axi_wready	: out std_logic;
		s00_axi_wstrb	: in std_logic_vector(3 downto 0);
		s00_axi_bresp	: out std_logic_vector(1 downto 0);
		s00_axi_bvalid	: out std_logic;
		s00_axi_bready	: in std_logic;
		s00_axi_araddr	: in std_logic_vector(C_S00_AXI_ADDR_WIDTH-1 downto 0);
		s00_axi_arprot  : in std_logic_vector(2 downto 0);
		s00_axi_arvalid	: in std_logic;
		s00_axi_arready	: out std_logic;
		s00_axi_rdata	: out std_logic_vector(C_S00_AXI_DATA_WIDTH-1 downto 0);
		s00_axi_rresp	: out std_logic_vector(1 downto 0);
		s00_axi_rvalid	: out std_logic;
		s00_axi_rready	: in std_logic
    );
end entity enseignement_addition_axi;

-----------------------------------------------------------------------
Architecture enseignement_addition_1 of enseignement_addition_axi is
-----------------------------------------------------------------------
	constant INTERNAL_ADDR_WIDTH : integer := 3;
	constant REG_ID		: std_logic_vector(2 downto 0) := "000";
	constant REG_OP1	: std_logic_vector(2 downto 0) := "001";
	constant REG_OP2	: std_logic_vector(2 downto 0) := "010";
	constant REG_RESULT	: std_logic_vector(2 downto 0) := "011";
	constant REG_TYPE_OP: std_logic_vector(2 downto 0) := "100";
    -- comm
    signal addr_s : std_logic_vector(INTERNAL_ADDR_WIDTH-1 downto 0);
    signal write_en_s, read_en_s : std_logic;

	signal readdata_s : std_logic_vector(C_S00_AXI_DATA_WIDTH-1 downto 0);
	signal op1_s, op2_s, result_s : std_logic_vector(C_S00_AXI_DATA_WIDTH-1 downto 0);
begin

	s00_axi_rdata <= readdata_s;

	result_s <= std_logic_vector(unsigned(op1_s)+unsigned(op2_s));

	-- manage register
	write_bloc : process(s00_axi_aclk, s00_axi_reset)
	begin
		 if s00_axi_reset = '1' then 
			 op1_s <= (others => '0');
			 op2_s <= (others => '0');
		 elsif rising_edge(s00_axi_aclk) then
			 op1_s <= op1_s;
			 op2_s <= op2_s;
		 end if;
	end process write_bloc;

	read_bloc : process(s00_axi_aclk, s00_axi_reset)
	begin
		if s00_axi_reset = '1' then
			readdata_s <= (others => '0');
		elsif rising_edge(s00_axi_aclk) then
			readdata_s <= readdata_s;
		end if;
	end process read_bloc;

	-- Instantiation of Axi Bus Interface S00_AXI
	handle_comm : entity work.add_handcomm
	generic map (C_S_AXI_DATA_WIDTH	=> C_S00_AXI_DATA_WIDTH,
		C_S_AXI_ADDR_WIDTH	=> C_S00_AXI_ADDR_WIDTH,
		INTERNAL_ADDR_WIDTH => INTERNAL_ADDR_WIDTH)
	port map (
		S_AXI_ACLK		=> s00_axi_aclk,
		S_AXI_RESET		=> s00_axi_reset,
		S_AXI_AWADDR	=> s00_axi_awaddr,
		S_AXI_AWVALID	=> s00_axi_awvalid,
		S_AXI_AWREADY	=> s00_axi_awready,
		S_AXI_WVALID	=> s00_axi_wvalid,
		S_AXI_WREADY	=> s00_axi_wready,
		S_AXI_BRESP	    => s00_axi_bresp,
		S_AXI_BVALID	=> s00_axi_bvalid,
		S_AXI_BREADY	=> s00_axi_bready,
		S_AXI_ARADDR	=> s00_axi_araddr,
		S_AXI_ARVALID	=> s00_axi_arvalid,
		S_AXI_ARREADY	=> s00_axi_arready,
		S_AXI_RRESP	    => s00_axi_rresp,
		S_AXI_RVALID	=> s00_axi_rvalid,
		S_AXI_RREADY	=> s00_axi_rready,
		read_en_o 		=> read_en_s,
		write_en_o 		=> write_en_s,
		addr_o 			=> addr_s);

end architecture enseignement_addition_1;

