library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.numeric_std.all;

Entity generateur_comm is 
    generic(
        id        : natural := 1;
		max_val		: natural := 715;
		AXI_DATA_WIDTH : natural := 32
    );
    port 
    (
		-- Syscon signals
		reset     : in std_logic;
		clk       : in std_logic;
		-- axi signals
		addr_i 		: in std_logic_vector(1 downto 0);
		write_en_i  : in std_logic;
		writedata   : in std_logic_vector(AXI_DATA_WIDTH-1 downto 0);
		read_en_i	: in std_logic;
		readdata    : out std_logic_vector(AXI_DATA_WIDTH-1 downto 0);
		-- 
		max_period_o : out std_logic_vector(AXI_DATA_WIDTH-1 downto 0)
    );
end entity generateur_comm;

-----------------------------------------------------------------------
Architecture generateur_comm_1 of generateur_comm is
-----------------------------------------------------------------------
	constant REG_ID     : std_logic_vector(1 downto 0) := "00";
	constant REG_PERIOD : std_logic_vector(1 downto 0) := "01";
	
	signal readdata_s : std_logic_vector(AXI_DATA_WIDTH-1 downto 0);
	signal max_period_s : std_logic_vector(AXI_DATA_WIDTH-1 downto 0);
begin
	max_period_o <= max_period_s;
	readdata <= readdata_s;

	-- manage register
	write_bloc : process(clk, reset)
	begin
		 if reset = '1' then 
			max_period_s <= std_logic_vector(to_unsigned(max_val, AXI_DATA_WIDTH));
		 elsif rising_edge(clk) then
			max_period_s <= max_period_s;
			if write_en_i = '1' then
				case addr_i is
				when REG_PERIOD =>
					max_period_s <= writedata;
				when others =>
				end case;
			end if;
		 end if;
	end process write_bloc;

	read_bloc : process(clk, reset)
	begin
		if reset = '1' then
			readdata_s <= (others => '0');
		elsif rising_edge(clk) then
			readdata_s <= readdata_s;
			if read_en_i = '1' then
				case addr_i is
				when REG_ID =>
					readdata_s <= std_logic_vector(to_unsigned(id,AXI_DATA_WIDTH));
				when REG_PERIOD =>
					readdata_s <= max_period_s;
				when others =>
					readdata_s <= (others => '1');  -- lecture dans reg inconnu
				end case;
			end if;
		end if;
	end process read_bloc;

end architecture generateur_comm_1;

