library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.numeric_std.all;

Entity generateur_logic is
generic (
    VECTOR_SIZE : natural := 32
);
port (
	rst_i		: in std_logic;
	clk_i		: in std_logic;
	output_o	: out std_logic;
	max_period_i: in std_logic_vector(VECTOR_SIZE-1 downto 0)
);
end entity;

---------------------------------------------------------------------------
Architecture generateur_logic_1 of generateur_logic is
---------------------------------------------------------------------------
	signal cpt_val_s, cpt_val_next : std_logic_vector(VECTOR_SIZE-1 downto 0);
	signal cpt_overflow_next	: std_logic;
	signal cpt_overflow_reg		: std_logic;
	signal max_s, mean_s		: std_logic_vector(VECTOR_SIZE-1 downto 0);
begin
	output_o <= cpt_overflow_reg;

	max_s <= std_logic_vector(unsigned(max_period_i)-1);
	mean_s<= std_logic_vector(unsigned('0'&max_period_i(VECTOR_SIZE-1 downto 1)));
	
	compteur : process(clk_i, rst_i)
	begin
		if rst_i = '1' then
			cpt_val_s <= (others => '0');
			cpt_overflow_reg <= '0';
		elsif rising_edge(clk_i) then
			cpt_val_s <= cpt_val_next;
			cpt_overflow_reg <= cpt_overflow_next;
		end if;
	end process;

	cpt_val_next <= (others => '0') when ((max_s <= cpt_val_s))
				else std_logic_vector(unsigned(cpt_val_s) + 1);
	cpt_overflow_next <= '1' when (cpt_val_s < mean_s) else '0';

end architecture generateur_logic_1;

