library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.numeric_std.all;

Entity periodcounter_comm is 
    generic(
        id        : natural := 1;
		AXI_DATA_WIDTH : natural := 32
    );
    port 
    (
		-- Syscon signals
		reset     : in std_logic;
		clk       : in std_logic;
		-- axi signals
		addr_i 		: in std_logic_vector(2 downto 0);
		write_en_i  : in std_logic;
		writedata   : in std_logic_vector(AXI_DATA_WIDTH-1 downto 0);
		read_en_i	: in std_logic;
		readdata    : out std_logic_vector(AXI_DATA_WIDTH-1 downto 0);
		-- 
		start_acquisition_o : out std_logic;
		busy_i : in std_logic;
		prescaler_o : out std_logic_vector(15 downto 0);
		--data
		data_val_i : in std_logic_vector(31 downto 0);
		data_addr_o : out std_logic_vector(9 downto 0)
    );
end entity periodcounter_comm;

-----------------------------------------------------------------------
Architecture periodcounter_comm_1 of periodcounter_comm is
-----------------------------------------------------------------------
	constant REG_ID     : std_logic_vector(2 downto 0) := "000";
	constant REG_STATUS : std_logic_vector(2 downto 0) := "001";
	constant REG_START	: std_logic_vector(2 downto 0) := "010";
	constant REG_DATA	: std_logic_vector(2 downto 0) := "011";
	constant REG_PRESC	: std_logic_vector(2 downto 0) := "100";

	signal data_addr_s : std_logic_vector(9 downto 0);
	signal readdata_s : std_logic_vector(AXI_DATA_WIDTH-1 downto 0);
	signal signal_old_s : std_logic;
	signal prescaler_s : std_logic_vector(15 downto 0);
begin
	data_addr_o <= data_addr_s;
	readdata <= readdata_s;
	prescaler_o <= prescaler_s;

	-- manage register
	write_bloc : process(clk, reset)
	begin
		 if reset = '1' then 
			start_acquisition_o <= '0';
			prescaler_s <= x"0000"; -- 1us
		 elsif rising_edge(clk) then
			start_acquisition_o <= '0';
			prescaler_s <= prescaler_s;
		 end if;
	end process write_bloc;

	read_bloc : process(clk, reset)
	begin
		if reset = '1' then
			readdata_s <= (others => '0');
			data_addr_s <= (others => '0');
		elsif rising_edge(clk) then
			data_addr_s <= data_addr_s;
			readdata_s <= readdata_s;
		end if;
	end process read_bloc;

end architecture periodcounter_comm_1;

