library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.numeric_std.all;

Entity periodcounter_logic is 
	port (
		-- Syscon signals
		reset_i		 : in std_logic;
		clk_i		   : in std_logic;
		-- signal to read
		input_sig_i : in std_logic;
		-- configuration prescaler
		prescaler_i : in std_logic_vector(15 downto 0);
		-- start
		start_i : in std_logic;
		busy_o  : out std_logic;
		-- ram storage
		ram_addr_o : out std_logic_vector(9 downto 0);
		ram_data_o : out std_logic_vector(31 downto 0);
		ram_en_o 	: out std_logic
	);
end entity;

---------------------------------------------------------------------------
Architecture periodcounter_logic_1 of periodcounter_logic is
	-- state machine
	type cpt_type is (IDLE, wait_first_rising, wait_rising);
	signal state_s : cpt_type;
	signal wr_ram_addr_s : std_logic_vector(9 downto 0);
	signal wr_ram_data_s : std_logic_vector(31 downto 0);
	signal wr_ram_wen_s  : std_logic;
	signal nb_receipt_s : natural range 0 to 2**10-1;
	--edge detector
	signal edge_s : std_logic;
	signal edge_detect_next_s, edge_detect_s: std_logic;
	--cpt period
	signal cpt_period_s, cpt_period_next_s : std_logic_vector(31 downto 0);
	--cpt prescaler
	signal max_cpt_presc_nat_s : natural range 0 to 2**16-1;
	signal cpt_prescaler_s, cpt_prescaler_next_s : natural range 0 to 2**16-1;
	signal tick2_s, tick_s, tick_next_s : std_logic;
	signal busy_s : std_logic;
	signal input_sig_s : std_logic;
begin
	ram_addr_o <= wr_ram_addr_s;
	ram_data_o <= wr_ram_data_s;
	ram_en_o   <= wr_ram_wen_s;
	busy_o <= busy_s;

	process(clk_i, reset_i)
	begin
		if reset_i = '1' then
			state_s <= IDLE;
			wr_ram_addr_s <= (others => '0');
			wr_ram_data_s <= (others => '0');
			wr_ram_wen_s <= '0';
			busy_s <= '0';
		elsif rising_edge(clk_i) then
			state_s <= state_s;
			wr_ram_addr_s <= wr_ram_addr_s;
			wr_ram_data_s <= wr_ram_data_s;
			wr_ram_wen_s <= '0';
			busy_s <= busy_s;

			case state_s is
			when IDLE => -- wait for a start from CPU
			when wait_first_rising =>	-- wait for a first edge
										-- avoid possible wrong
										-- start value
			when wait_rising =>			-- store until nb_receipt_s == MAX
			end case;
		end if;
	end process;

	debounce_inst : Entity work.periodcounter_debounce
	generic map ( NB_CYCLE => 4)
	port map (
		-- Syscon signals
		reset	=> reset_i,
		clk		=> clk_i,
		input_signal => input_sig_i,
		rising_signal 	=> edge_detect_s,
		falling_signal	=> open);

end architecture periodcounter_logic_1;

